mod db;
mod server;

use std::sync::Arc;

use r2d2_sqlite::SqliteConnectionManager;
use tera::Tera;
use tower_http::compression::CompressionLayer;

const MIGRATIONS: [&str; 4] = [
    include_str!("migrations/000_initial.sql"),
    include_str!("migrations/001_plants.sql"),
    include_str!("migrations/002_comments.sql"),
    include_str!("migrations/003_events.sql"),
];
const STATIC_FILES: [(&str, &'static str); 1] = [(
    "htmx_2.0.4.min.js",
    include_str!("static/htmx_2.0.4.min.js"),
)];

#[derive(Clone)]
pub struct Context {
    pool: db::DbPool,
    tera: Arc<Tera>,
}

#[tokio::main]
async fn main() {
    tracing_subscriber::fmt::init();

    let manager = SqliteConnectionManager::file("db.sqlite");
    let pool = r2d2::Pool::new(manager).unwrap();
    db::run_migrations(&pool, &MIGRATIONS).unwrap();

    let template_dir = std::env::var("TEMPLATE_DIR").unwrap_or(String::from("./templates"));
    let tera = Tera::new(&format!("{template_dir}/**/*")).unwrap();

    let ctx = Context {
        pool,
        tera: Arc::new(tera),
    };
    let app = server::app(ctx).layer(CompressionLayer::new().br(true).gzip(true));

    let address = "0.0.0.0:8000";

    tracing::info!("Starting server on {address}");

    let listener = tokio::net::TcpListener::bind(address).await.unwrap();
    axum::serve(listener, app.into_make_service())
        .await
        .unwrap();
}
